#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <vector>
#include <set>
using namespace std;

const int max_n = 5000;

vector <vector <int> > g;
vector <int> status; // 0 - radius station,
					// 1 - ring station
					// 2 - kremlin
int n, m;

vector <int> dsu;

int get(int a)
{
	if (dsu[a] != a) {
		return dsu[a] = get(dsu[a]);
	} else {
		return dsu[a];
	}
}

void connect(int a, int b)
{
	a = get(a);
	b = get(b);
	dsu[a] = b;
}

void bad_map()
{
	cout << "No\n";
	exit(0);
}

int main()
{
	freopen("map.in", "r", stdin);
	freopen("map.out","w", stdout);

	ios::sync_with_stdio(false);
	cin >> n >> m;
	g.resize(n);
	status.resize(n);
	for (int i = 0; i < m; ++i) {
		int a, b;
		cin >> a >> b;
		--a, --b;
		g[a].push_back(b);
		g[b].push_back(a);
	}
	//----set status--------------
	for (int i = 0; i < n; ++i) {
		if (g[i].size() >= 5) {
			status[i] = 2;
		} else if (g[i].size() == 4) {
			status[i] = 1;
		} else if (g[i].size() == 3) {
			bad_map();
		}
	}
	//----check kremlin-----------
	int kremlin = 0;
	int ring_example = 0;
	int kremlin_cnt = 0;
	int ring_cnt = 0;

	for (int i = 0; i < n; ++i) {
		if (status[i] == 2) {
			kremlin_cnt++;
			kremlin = i;
		} else if (status[i] == 1) {
			ring_cnt++;
			ring_example = i;
		}
	}
	if (kremlin_cnt != 1) {
		bad_map();
	}
	if (ring_cnt != g[kremlin].size()) {
		bad_map();
	}
	cerr << "One kremlin: OK\n";
	cerr << "All ring-st are usefull: OK\n";
	
	//----check connection-------- 
    dsu.resize(n); 
    for (int i = 0; i < n; ++i) { 
        dsu[i] = i; 
    } 
    for (int v = 0; v < n; ++v) { 
        for (int u = 0; u < g[v].size(); ++u) { 
            if (status[v] != 1 || status[g[v][u]] != 1) { 
                connect(v, g[v][u]); 
            } 
        } 
    } 
    for (int v = 0; v < n; ++v) { 
        if (get(v) != get(0)) { 
            bad_map(); 
        } 
    } 
    cerr << "Map is connected: OK\n"; 
	
	//----one cycle check---------
	for (int v = 0; v < n; ++v) {
		if (status[v] != 1) {
			continue;
		}
		int ring_neig_cnt = 0;
		for (int i = 0; i < g[v].size(); ++i) {
			ring_neig_cnt += (status[g[v][i]] == 1);
		}
		if (ring_neig_cnt != 2) {
			bad_map();
		}
	}
	cerr << "Ring in cycle: OK\n";

	//----check single ring-------
	dsu.resize(n);
	for (int i = 0; i < n; ++i) {
		dsu[i] = i;
	}
	for (int v = 0; v < n; ++v) {
		for (int u = 0; u < g[v].size(); ++u) {
			if (status[v] == 1 && status[g[v][u]] == 1) {
				connect(v, g[v][u]);
			}
		}
	}
	for (int v = 0; v < n; ++v) {
		if (status[v] == 1 && get(v) != get(ring_example)) {
			bad_map();
		}
	}
	cerr << "Ring is connected: OK\n";

	//----check second ring-------
	dsu.resize(n);
	for (int i = 0; i < n; ++i) {
		dsu[i] = i;
	}
	for (int v = 0; v < n; ++v) {
		for (int u = 0; u < g[v].size(); ++u) {
			if (status[v] + status[g[v][u]] <= 1) {
				connect(v, g[v][u]);
			}
		}
	}
	set <int> neig_roots;
	set <int> ring_roots;
	for (int i = 0; i < n; ++i) {
		if (status[i] == 1) {
			ring_roots.insert(get(i));
		}
	}
	if (g[kremlin].size() != ring_roots.size()) {
		bad_map();
	}
	for (int i = 0; i < g[kremlin].size(); ++i) {
		neig_roots.insert(get(g[kremlin][i]));
	}
	if (g[kremlin].size() != neig_roots.size()) {
		bad_map();
	}
	cerr << "No outside connections: OK\n";
	//----Ok----------------------
	cout << "Yes\n";
	return 0;
}
